# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/tool.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1274, 1096)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        MainWindow.setFont(font)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.scrollArea = QtWidgets.QScrollArea(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollArea.sizePolicy().hasHeightForWidth())
        self.scrollArea.setSizePolicy(sizePolicy)
        self.scrollArea.setMaximumSize(QtCore.QSize(385, 16777215))
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 362, 1476))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollAreaWidgetContents.sizePolicy().hasHeightForWidth())
        self.scrollAreaWidgetContents.setSizePolicy(sizePolicy)
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout(self.scrollAreaWidgetContents)
        self.horizontalLayout_17.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_17.setSpacing(0)
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.widget = QtWidgets.QWidget(self.scrollAreaWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget.sizePolicy().hasHeightForWidth())
        self.widget.setSizePolicy(sizePolicy)
        self.widget.setObjectName("widget")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout_4.setContentsMargins(5, 5, 5, 5)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label_6 = QtWidgets.QLabel(self.widget)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(11)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_4.addWidget(self.label_6)
        self.line_3 = QtWidgets.QFrame(self.widget)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Raised)
        self.line_3.setLineWidth(1)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setObjectName("line_3")
        self.verticalLayout_4.addWidget(self.line_3)
        self.widget_4 = QtWidgets.QWidget(self.widget)
        self.widget_4.setObjectName("widget_4")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.widget_4)
        self.verticalLayout_9.setContentsMargins(-1, 0, 0, 0)
        self.verticalLayout_9.setSpacing(4)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pb_loadFile = QtWidgets.QPushButton(self.widget_4)
        self.pb_loadFile.setObjectName("pb_loadFile")
        self.horizontalLayout.addWidget(self.pb_loadFile)
        self.le_loadFile = QtWidgets.QLineEdit(self.widget_4)
        self.le_loadFile.setReadOnly(True)
        self.le_loadFile.setObjectName("le_loadFile")
        self.horizontalLayout.addWidget(self.le_loadFile)
        self.verticalLayout_9.addLayout(self.horizontalLayout)
        self.groupBox = QtWidgets.QGroupBox(self.widget_4)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.comb_fieldX = QtWidgets.QComboBox(self.groupBox)
        self.comb_fieldX.setObjectName("comb_fieldX")
        self.horizontalLayout_2.addWidget(self.comb_fieldX)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.comb_fieldY = QtWidgets.QComboBox(self.groupBox)
        self.comb_fieldY.setObjectName("comb_fieldY")
        self.horizontalLayout_3.addWidget(self.comb_fieldY)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_4.addWidget(self.label_3)
        self.comb_fieldZ = QtWidgets.QComboBox(self.groupBox)
        self.comb_fieldZ.setObjectName("comb_fieldZ")
        self.horizontalLayout_4.addWidget(self.comb_fieldZ)
        self.check_fieldZ = QtWidgets.QCheckBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.check_fieldZ.sizePolicy().hasHeightForWidth())
        self.check_fieldZ.setSizePolicy(sizePolicy)
        self.check_fieldZ.setText("")
        self.check_fieldZ.setObjectName("check_fieldZ")
        self.horizontalLayout_4.addWidget(self.check_fieldZ)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_5.addWidget(self.label_4)
        self.comb_fieldP = QtWidgets.QComboBox(self.groupBox)
        self.comb_fieldP.setObjectName("comb_fieldP")
        self.horizontalLayout_5.addWidget(self.comb_fieldP)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.pb_showPoints = QtWidgets.QPushButton(self.groupBox)
        self.pb_showPoints.setObjectName("pb_showPoints")
        self.verticalLayout.addWidget(self.pb_showPoints)
        self.verticalLayout_9.addWidget(self.groupBox)
        self.verticalLayout_4.addWidget(self.widget_4)
        self.label_7 = QtWidgets.QLabel(self.widget)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(10)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_4.addWidget(self.label_7)
        self.line = QtWidgets.QFrame(self.widget)
        self.line.setFrameShadow(QtWidgets.QFrame.Raised)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout_4.addWidget(self.line)
        self.widget_3 = QtWidgets.QWidget(self.widget)
        self.widget_3.setObjectName("widget_3")
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout(self.widget_3)
        self.horizontalLayout_16.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.pb_calcVariogram = QtWidgets.QPushButton(self.widget_3)
        self.pb_calcVariogram.setObjectName("pb_calcVariogram")
        self.verticalLayout_3.addWidget(self.pb_calcVariogram)
        self.groupBox_2 = QtWidgets.QGroupBox(self.widget_3)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.rb_calcVariogramDefault = QtWidgets.QRadioButton(self.groupBox_2)
        self.rb_calcVariogramDefault.setChecked(True)
        self.rb_calcVariogramDefault.setObjectName("rb_calcVariogramDefault")
        self.verticalLayout_11.addWidget(self.rb_calcVariogramDefault)
        self.rb_calcVariogramCustom = QtWidgets.QRadioButton(self.groupBox_2)
        self.rb_calcVariogramCustom.setEnabled(True)
        self.rb_calcVariogramCustom.setObjectName("rb_calcVariogramCustom")
        self.verticalLayout_11.addWidget(self.rb_calcVariogramCustom)
        self.rb_calcVariogramOmni = QtWidgets.QRadioButton(self.groupBox_2)
        self.rb_calcVariogramOmni.setEnabled(True)
        self.rb_calcVariogramOmni.setObjectName("rb_calcVariogramOmni")
        self.verticalLayout_11.addWidget(self.rb_calcVariogramOmni)
        self.verticalLayout_3.addWidget(self.groupBox_2)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_5 = QtWidgets.QLabel(self.widget_3)
        self.label_5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_14.addWidget(self.label_5)
        self.comb_variogramModel = QtWidgets.QComboBox(self.widget_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comb_variogramModel.sizePolicy().hasHeightForWidth())
        self.comb_variogramModel.setSizePolicy(sizePolicy)
        self.comb_variogramModel.setObjectName("comb_variogramModel")
        self.horizontalLayout_14.addWidget(self.comb_variogramModel)
        self.horizontalLayout_14.setStretch(0, 1)
        self.horizontalLayout_14.setStretch(1, 2)
        self.verticalLayout_3.addLayout(self.horizontalLayout_14)
        self.gb_customVariogram = QtWidgets.QGroupBox(self.widget_3)
        self.gb_customVariogram.setCheckable(False)
        self.gb_customVariogram.setChecked(False)
        self.gb_customVariogram.setObjectName("gb_customVariogram")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.gb_customVariogram)
        self.verticalLayout_2.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.label_lagInterval = QtWidgets.QLabel(self.gb_customVariogram)
        self.label_lagInterval.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_lagInterval.setObjectName("label_lagInterval")
        self.verticalLayout_6.addWidget(self.label_lagInterval)
        self.label_lagNum = QtWidgets.QLabel(self.gb_customVariogram)
        self.label_lagNum.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_lagNum.setObjectName("label_lagNum")
        self.verticalLayout_6.addWidget(self.label_lagNum)
        self.label_lagTole = QtWidgets.QLabel(self.gb_customVariogram)
        self.label_lagTole.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_lagTole.setObjectName("label_lagTole")
        self.verticalLayout_6.addWidget(self.label_lagTole)
        self.label_angle = QtWidgets.QLabel(self.gb_customVariogram)
        self.label_angle.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_angle.setObjectName("label_angle")
        self.verticalLayout_6.addWidget(self.label_angle)
        self.label_angleDip = QtWidgets.QLabel(self.gb_customVariogram)
        self.label_angleDip.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_angleDip.setObjectName("label_angleDip")
        self.verticalLayout_6.addWidget(self.label_angleDip)
        self.label_angleTole = QtWidgets.QLabel(self.gb_customVariogram)
        self.label_angleTole.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_angleTole.setObjectName("label_angleTole")
        self.verticalLayout_6.addWidget(self.label_angleTole)
        self.label_bandWidth = QtWidgets.QLabel(self.gb_customVariogram)
        self.label_bandWidth.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_bandWidth.setObjectName("label_bandWidth")
        self.verticalLayout_6.addWidget(self.label_bandWidth)
        self.horizontalLayout_6.addLayout(self.verticalLayout_6)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.le_lagInterval = QtWidgets.QLineEdit(self.gb_customVariogram)
        self.le_lagInterval.setObjectName("le_lagInterval")
        self.verticalLayout_5.addWidget(self.le_lagInterval)
        self.le_lagNum = QtWidgets.QLineEdit(self.gb_customVariogram)
        self.le_lagNum.setObjectName("le_lagNum")
        self.verticalLayout_5.addWidget(self.le_lagNum)
        self.le_lagTole = QtWidgets.QLineEdit(self.gb_customVariogram)
        self.le_lagTole.setObjectName("le_lagTole")
        self.verticalLayout_5.addWidget(self.le_lagTole)
        self.le_angle = QtWidgets.QLineEdit(self.gb_customVariogram)
        self.le_angle.setObjectName("le_angle")
        self.verticalLayout_5.addWidget(self.le_angle)
        self.le_angleDip = QtWidgets.QLineEdit(self.gb_customVariogram)
        self.le_angleDip.setObjectName("le_angleDip")
        self.verticalLayout_5.addWidget(self.le_angleDip)
        self.le_angleTole = QtWidgets.QLineEdit(self.gb_customVariogram)
        self.le_angleTole.setObjectName("le_angleTole")
        self.verticalLayout_5.addWidget(self.le_angleTole)
        self.le_bandWidth = QtWidgets.QLineEdit(self.gb_customVariogram)
        self.le_bandWidth.setObjectName("le_bandWidth")
        self.verticalLayout_5.addWidget(self.le_bandWidth)
        self.horizontalLayout_6.addLayout(self.verticalLayout_5)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.verticalLayout_3.addWidget(self.gb_customVariogram)
        self.horizontalLayout_16.addLayout(self.verticalLayout_3)
        self.verticalLayout_4.addWidget(self.widget_3)
        self.label_8 = QtWidgets.QLabel(self.widget)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(10)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.verticalLayout_4.addWidget(self.label_8)
        self.line_2 = QtWidgets.QFrame(self.widget)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_4.addWidget(self.line_2)
        self.widget_2 = QtWidgets.QWidget(self.widget)
        self.widget_2.setObjectName("widget_2")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.widget_2)
        self.verticalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.pb_executeInterp = QtWidgets.QPushButton(self.widget_2)
        self.pb_executeInterp.setObjectName("pb_executeInterp")
        self.verticalLayout_7.addWidget(self.pb_executeInterp)
        self.gb_mode = QtWidgets.QGroupBox(self.widget_2)
        self.gb_mode.setFlat(True)
        self.gb_mode.setCheckable(False)
        self.gb_mode.setChecked(False)
        self.gb_mode.setObjectName("gb_mode")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout(self.gb_mode)
        self.horizontalLayout_10.setContentsMargins(4, 4, 4, 4)
        self.horizontalLayout_10.setSpacing(3)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.rb_interp = QtWidgets.QRadioButton(self.gb_mode)
        self.rb_interp.setChecked(True)
        self.rb_interp.setObjectName("rb_interp")
        self.verticalLayout_8.addWidget(self.rb_interp)
        self.rb_cv = QtWidgets.QRadioButton(self.gb_mode)
        self.rb_cv.setObjectName("rb_cv")
        self.verticalLayout_8.addWidget(self.rb_cv)
        self.horizontalLayout_10.addLayout(self.verticalLayout_8)
        self.verticalLayout_7.addWidget(self.gb_mode)
        self.gb_grid = QtWidgets.QGroupBox(self.widget_2)
        self.gb_grid.setFlat(True)
        self.gb_grid.setObjectName("gb_grid")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.gb_grid)
        self.verticalLayout_10.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.le_ez = QtWidgets.QLineEdit(self.gb_grid)
        self.le_ez.setEnabled(False)
        self.le_ez.setMaximumSize(QtCore.QSize(60, 16777215))
        self.le_ez.setObjectName("le_ez")
        self.gridLayout.addWidget(self.le_ez, 2, 3, 1, 1)
        self.le_ey = QtWidgets.QLineEdit(self.gb_grid)
        self.le_ey.setMaximumSize(QtCore.QSize(60, 16777215))
        self.le_ey.setObjectName("le_ey")
        self.gridLayout.addWidget(self.le_ey, 2, 2, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.gb_grid)
        self.label_10.setAlignment(QtCore.Qt.AlignCenter)
        self.label_10.setObjectName("label_10")
        self.gridLayout.addWidget(self.label_10, 0, 2, 1, 1)
        self.le_by = QtWidgets.QLineEdit(self.gb_grid)
        self.le_by.setMaximumSize(QtCore.QSize(60, 16777215))
        self.le_by.setObjectName("le_by")
        self.gridLayout.addWidget(self.le_by, 1, 2, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.gb_grid)
        self.label_14.setAlignment(QtCore.Qt.AlignCenter)
        self.label_14.setObjectName("label_14")
        self.gridLayout.addWidget(self.label_14, 0, 3, 1, 1)
        self.le_bx = QtWidgets.QLineEdit(self.gb_grid)
        self.le_bx.setMaximumSize(QtCore.QSize(60, 16777215))
        self.le_bx.setObjectName("le_bx")
        self.gridLayout.addWidget(self.le_bx, 1, 1, 1, 1)
        self.le_bz = QtWidgets.QLineEdit(self.gb_grid)
        self.le_bz.setEnabled(False)
        self.le_bz.setMaximumSize(QtCore.QSize(60, 16777215))
        self.le_bz.setObjectName("le_bz")
        self.gridLayout.addWidget(self.le_bz, 1, 3, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.gb_grid)
        self.label_9.setAlignment(QtCore.Qt.AlignCenter)
        self.label_9.setObjectName("label_9")
        self.gridLayout.addWidget(self.label_9, 0, 1, 1, 1)
        self.label_16 = QtWidgets.QLabel(self.gb_grid)
        self.label_16.setAlignment(QtCore.Qt.AlignCenter)
        self.label_16.setObjectName("label_16")
        self.gridLayout.addWidget(self.label_16, 1, 0, 1, 1)
        self.le_ex = QtWidgets.QLineEdit(self.gb_grid)
        self.le_ex.setMaximumSize(QtCore.QSize(60, 16777215))
        self.le_ex.setObjectName("le_ex")
        self.gridLayout.addWidget(self.le_ex, 2, 1, 1, 1)
        self.le_nx = QtWidgets.QLineEdit(self.gb_grid)
        self.le_nx.setMaximumSize(QtCore.QSize(60, 16777215))
        self.le_nx.setObjectName("le_nx")
        self.gridLayout.addWidget(self.le_nx, 3, 1, 1, 1)
        self.le_ny = QtWidgets.QLineEdit(self.gb_grid)
        self.le_ny.setMaximumSize(QtCore.QSize(60, 16777215))
        self.le_ny.setObjectName("le_ny")
        self.gridLayout.addWidget(self.le_ny, 3, 2, 1, 1)
        self.le_nz = QtWidgets.QLineEdit(self.gb_grid)
        self.le_nz.setEnabled(False)
        self.le_nz.setMaximumSize(QtCore.QSize(60, 16777215))
        self.le_nz.setObjectName("le_nz")
        self.gridLayout.addWidget(self.le_nz, 3, 3, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.gb_grid)
        self.label_17.setAlignment(QtCore.Qt.AlignCenter)
        self.label_17.setObjectName("label_17")
        self.gridLayout.addWidget(self.label_17, 2, 0, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.gb_grid)
        self.label_18.setAlignment(QtCore.Qt.AlignCenter)
        self.label_18.setObjectName("label_18")
        self.gridLayout.addWidget(self.label_18, 3, 0, 1, 1)
        self.verticalLayout_10.addLayout(self.gridLayout)
        self.verticalLayout_7.addWidget(self.gb_grid)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_11 = QtWidgets.QLabel(self.widget_2)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_12.addWidget(self.label_11)
        self.comb_method = QtWidgets.QComboBox(self.widget_2)
        self.comb_method.setObjectName("comb_method")
        self.horizontalLayout_12.addWidget(self.comb_method)
        self.verticalLayout_7.addLayout(self.horizontalLayout_12)
        self.gb_variogram = QtWidgets.QGroupBox(self.widget_2)
        self.gb_variogram.setObjectName("gb_variogram")
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout(self.gb_variogram)
        self.horizontalLayout_15.setContentsMargins(4, 4, 4, 4)
        self.horizontalLayout_15.setSpacing(4)
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.rb_variogramNone = QtWidgets.QRadioButton(self.gb_variogram)
        self.rb_variogramNone.setObjectName("rb_variogramNone")
        self.horizontalLayout_15.addWidget(self.rb_variogramNone)
        self.rb_variogramDefault = QtWidgets.QRadioButton(self.gb_variogram)
        self.rb_variogramDefault.setChecked(True)
        self.rb_variogramDefault.setObjectName("rb_variogramDefault")
        self.horizontalLayout_15.addWidget(self.rb_variogramDefault)
        self.rb_variogramCalced = QtWidgets.QRadioButton(self.gb_variogram)
        self.rb_variogramCalced.setEnabled(False)
        self.rb_variogramCalced.setObjectName("rb_variogramCalced")
        self.horizontalLayout_15.addWidget(self.rb_variogramCalced)
        self.verticalLayout_7.addWidget(self.gb_variogram)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_12 = QtWidgets.QLabel(self.widget_2)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_7.addWidget(self.label_12)
        self.sb_neighSize = QtWidgets.QSpinBox(self.widget_2)
        self.sb_neighSize.setMinimum(2)
        self.sb_neighSize.setMaximum(99)
        self.sb_neighSize.setProperty("value", 8)
        self.sb_neighSize.setObjectName("sb_neighSize")
        self.horizontalLayout_7.addWidget(self.sb_neighSize)
        self.verticalLayout_7.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_alpha = QtWidgets.QLabel(self.widget_2)
        self.label_alpha.setObjectName("label_alpha")
        self.horizontalLayout_8.addWidget(self.label_alpha)
        self.le_alpha = QtWidgets.QLineEdit(self.widget_2)
        self.le_alpha.setObjectName("le_alpha")
        self.horizontalLayout_8.addWidget(self.le_alpha)
        self.verticalLayout_7.addLayout(self.horizontalLayout_8)
        self.gb_useTF = QtWidgets.QGroupBox(self.widget_2)
        self.gb_useTF.setFlat(True)
        self.gb_useTF.setCheckable(True)
        self.gb_useTF.setObjectName("gb_useTF")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(self.gb_useTF)
        self.horizontalLayout_9.setContentsMargins(4, 4, 4, 4)
        self.horizontalLayout_9.setSpacing(4)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_13 = QtWidgets.QLabel(self.gb_useTF)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_13.addWidget(self.label_13)
        self.comb_device = QtWidgets.QComboBox(self.gb_useTF)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comb_device.sizePolicy().hasHeightForWidth())
        self.comb_device.setSizePolicy(sizePolicy)
        self.comb_device.setObjectName("comb_device")
        self.horizontalLayout_13.addWidget(self.comb_device)
        self.horizontalLayout_9.addLayout(self.horizontalLayout_13)
        self.verticalLayout_7.addWidget(self.gb_useTF)
        self.verticalLayout_4.addWidget(self.widget_2)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.pb_savePs = QtWidgets.QPushButton(self.widget)
        self.pb_savePs.setObjectName("pb_savePs")
        self.horizontalLayout_11.addWidget(self.pb_savePs)
        self.pb_saveGrid = QtWidgets.QPushButton(self.widget)
        self.pb_saveGrid.setObjectName("pb_saveGrid")
        self.horizontalLayout_11.addWidget(self.pb_saveGrid)
        self.verticalLayout_4.addLayout(self.horizontalLayout_11)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem)
        self.horizontalLayout_17.addWidget(self.widget)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.horizontalLayout_18.addWidget(self.scrollArea)
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_interp = QtWidgets.QWidget()
        self.tab_interp.setObjectName("tab_interp")
        self.tabWidget.addTab(self.tab_interp, "")
        self.tab_variogram = QtWidgets.QWidget()
        self.tab_variogram.setObjectName("tab_variogram")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.tab_variogram)
        self.verticalLayout_12.setContentsMargins(5, 5, 5, 5)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.scrollArea_2 = QtWidgets.QScrollArea(self.tab_variogram)
        self.scrollArea_2.setWidgetResizable(True)
        self.scrollArea_2.setObjectName("scrollArea_2")
        self.scrollAreaWidgetContents_2 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_2.setGeometry(QtCore.QRect(0, 0, 842, 1000))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollAreaWidgetContents_2.sizePolicy().hasHeightForWidth())
        self.scrollAreaWidgetContents_2.setSizePolicy(sizePolicy)
        self.scrollAreaWidgetContents_2.setObjectName("scrollAreaWidgetContents_2")
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout(self.scrollAreaWidgetContents_2)
        self.horizontalLayout_19.setContentsMargins(4, 4, 4, 4)
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.vl_variogram = QtWidgets.QVBoxLayout()
        self.vl_variogram.setObjectName("vl_variogram")
        self.horizontalLayout_19.addLayout(self.vl_variogram)
        self.scrollArea_2.setWidget(self.scrollAreaWidgetContents_2)
        self.verticalLayout_12.addWidget(self.scrollArea_2)
        self.tabWidget.addTab(self.tab_variogram, "")
        self.tab_cv = QtWidgets.QWidget()
        self.tab_cv.setObjectName("tab_cv")
        self.tabWidget.addTab(self.tab_cv, "")
        self.horizontalLayout_18.addWidget(self.tabWidget)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusBar = QtWidgets.QStatusBar(MainWindow)
        self.statusBar.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.statusBar.setObjectName("statusBar")
        MainWindow.setStatusBar(self.statusBar)

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "TFInterpy Gui Tool"))
        self.label_6.setText(_translate("MainWindow", "File"))
        self.pb_loadFile.setText(_translate("MainWindow", "load file"))
        self.groupBox.setTitle(_translate("MainWindow", "fields"))
        self.label.setText(_translate("MainWindow", "X"))
        self.label_2.setText(_translate("MainWindow", "Y"))
        self.label_3.setText(_translate("MainWindow", "Z"))
        self.label_4.setText(_translate("MainWindow", "P"))
        self.pb_showPoints.setText(_translate("MainWindow", "show points"))
        self.label_7.setText(_translate("MainWindow", "Variogram"))
        self.pb_calcVariogram.setText(_translate("MainWindow", "calculate variogram"))
        self.groupBox_2.setTitle(_translate("MainWindow", "mode"))
        self.rb_calcVariogramDefault.setText(_translate("MainWindow", "default"))
        self.rb_calcVariogramCustom.setText(_translate("MainWindow", "custom"))
        self.rb_calcVariogramOmni.setText(_translate("MainWindow", "omnidirectional"))
        self.label_5.setText(_translate("MainWindow", "variogram model"))
        self.gb_customVariogram.setTitle(_translate("MainWindow", "custom lags search"))
        self.label_lagInterval.setText(_translate("MainWindow", "lag interval"))
        self.label_lagNum.setText(_translate("MainWindow", "lag number"))
        self.label_lagTole.setText(_translate("MainWindow", "lag tolerent"))
        self.label_angle.setText(_translate("MainWindow", "angle"))
        self.label_angleDip.setText(_translate("MainWindow", "dip angle"))
        self.label_angleTole.setText(_translate("MainWindow", "angle tolerent"))
        self.label_bandWidth.setText(_translate("MainWindow", "band width"))
        self.le_angle.setPlaceholderText(_translate("MainWindow", "[0-PI]"))
        self.le_angleDip.setPlaceholderText(_translate("MainWindow", "[0-PI]"))
        self.label_8.setText(_translate("MainWindow", "Interpolation"))
        self.pb_executeInterp.setText(_translate("MainWindow", "execute"))
        self.gb_mode.setTitle(_translate("MainWindow", "mode"))
        self.rb_interp.setText(_translate("MainWindow", "interpolate"))
        self.rb_cv.setText(_translate("MainWindow", "cross validate"))
        self.gb_grid.setTitle(_translate("MainWindow", "grid"))
        self.label_10.setText(_translate("MainWindow", "Y"))
        self.label_14.setText(_translate("MainWindow", "Z"))
        self.label_9.setText(_translate("MainWindow", "X"))
        self.label_16.setText(_translate("MainWindow", "begin"))
        self.label_17.setText(_translate("MainWindow", "end"))
        self.label_18.setText(_translate("MainWindow", "number"))
        self.label_11.setText(_translate("MainWindow", "method"))
        self.gb_variogram.setTitle(_translate("MainWindow", "variogram"))
        self.rb_variogramNone.setText(_translate("MainWindow", "none"))
        self.rb_variogramDefault.setText(_translate("MainWindow", "default"))
        self.rb_variogramCalced.setText(_translate("MainWindow", "caculated"))
        self.label_12.setText(_translate("MainWindow", "neighborhood size"))
        self.label_alpha.setText(_translate("MainWindow", "alpha"))
        self.le_alpha.setText(_translate("MainWindow", "2"))
        self.gb_useTF.setTitle(_translate("MainWindow", "use tensorflow"))
        self.label_13.setText(_translate("MainWindow", "device"))
        self.pb_savePs.setText(_translate("MainWindow", "save points"))
        self.pb_saveGrid.setText(_translate("MainWindow", "save grid"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_interp), _translate("MainWindow", "interpolation"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_variogram), _translate("MainWindow", "variogram"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_cv), _translate("MainWindow", "cross validate"))

